@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2001 Havoc Pennington, 2011-2012 Andreas Rönnquist
+ *	Copyright (C) 2001 Havoc Pennington, 2011-2013 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -31,10 +31,6 @@
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
 
-
-#define WNCK_I_KNOW_THIS_IS_UNSTABLE
-#include <libwnck/libwnck.h>
-
 #include "xutils.h"
 
 
@@ -85,7 +81,7 @@ Atom my_wnck_atom_get(const char *atom_name)
 /**
  *
  */
-void my_wnck_change_state(Screen *screen, Window xwindow,
+void devilspie2_change_state(Screen *screen, Window xwindow,
 							gboolean add,
 							Atom     state1,
 							Atom     state2)
@@ -118,7 +114,7 @@ void my_wnck_change_state(Screen *screen, Window xwindow,
 /**
  *
  */
-void my_wnck_error_trap_push()
+void devilspie2_error_trap_push()
 {
 	gdk_error_trap_push();
 }
@@ -127,7 +123,7 @@ void my_wnck_error_trap_push()
 /**
  *
  */
-int my_wnck_error_trap_pop()
+int devilspie2_error_trap_pop()
 {
 	XSync(gdk_x11_get_default_xdisplay(),False);
 	return gdk_error_trap_pop();
@@ -137,7 +133,7 @@ int my_wnck_error_trap_pop()
 /**
  *
  */
-static void set_decorations(WnckWindow *window, gboolean decorate)
+static void set_decorations(Window xid /*WnckWindow *window*/, gboolean decorate)
 {
 #define PROP_MOTIF_WM_HINTS_ELEMENTS 5
 #define MWM_HINTS_DECORATIONS (1L << 1)
@@ -153,15 +149,22 @@ static void set_decorations(WnckWindow *window, gboolean decorate)
 	hints.decorations = decorate ? 1 : 0;
 
 	/* Set Motif hints, most window managers handle these */
-	XChangeProperty(gdk_x11_get_default_xdisplay(), wnck_window_get_xid (window),
+	XChangeProperty(gdk_x11_get_default_xdisplay(), xid /*wnck_window_get_xid (window)*/,
 						my_wnck_atom_get ("_MOTIF_WM_HINTS"),
 						my_wnck_atom_get ("_MOTIF_WM_HINTS"), 32, PropModeReplace,
 						(unsigned char *)&hints, PROP_MOTIF_WM_HINTS_ELEMENTS);
 
+
+	   //Window   xid;
+   XWindowAttributes attrs;
+
+   //xid = wnck_window_get_xid (window);
+   XGetWindowAttributes(gdk_x11_get_default_xdisplay(), xid, &attrs);
+
 	/* Apart from OpenBox, which doesn't respect it changing after mapping.
 	  Instead it has this workaround. */
-	my_wnck_change_state (my_wnck_window_get_xscreen(window),
-								wnck_window_get_xid(window), !decorate,
+	devilspie2_change_state (attrs.screen,
+								xid /*wnck_window_get_xid(window)*/, !decorate,
 								my_wnck_atom_get ("_OB_WM_STATE_UNDECORATED"), 0);
 
 }
@@ -170,13 +173,13 @@ static void set_decorations(WnckWindow *window, gboolean decorate)
 /**
  *
  */
-gboolean decorate_window(WnckWindow *window)
+gboolean decorate_window(Window xid)
 {
-	my_wnck_error_trap_push();
+	devilspie2_error_trap_push();
 
-	set_decorations(window,TRUE);
+	set_decorations(xid, TRUE);
 
-	if (my_wnck_error_trap_pop()) {
+	if (devilspie2_error_trap_pop()) {
 		g_printerr("decorate_window %s\n",_("Failed!"));
 		return FALSE;
 	}
@@ -188,13 +191,13 @@ gboolean decorate_window(WnckWindow *window)
 /**
  *
  */
-gboolean undecorate_window(WnckWindow *window)
+gboolean undecorate_window(Window xid)
 {
-	my_wnck_error_trap_push();
+	devilspie2_error_trap_push();
 
-	set_decorations(window,FALSE);
+	set_decorations(xid, FALSE);
 
-	if (my_wnck_error_trap_pop()) {
+	if (devilspie2_error_trap_pop()) {
 		g_printerr("decorate_window %s\n", _("Failed!"));
 		return FALSE;
 	}
@@ -206,12 +209,10 @@ gboolean undecorate_window(WnckWindow *window)
 /**
  *
  */
-Screen *my_wnck_window_get_xscreen(WnckWindow *window)
+Screen *devilspie2_window_get_xscreen(Window xid)
 {
-   Window   xid;
    XWindowAttributes attrs;
 
-   xid = wnck_window_get_xid (window);
    XGetWindowAttributes(gdk_x11_get_default_xdisplay(), xid, &attrs);
 
    return attrs.screen;
@@ -231,15 +232,16 @@ char* my_wnck_get_string_property_latin1(Window xwindow, Atom atom)
 	int err, result;
 	char *retval;
 
-	my_wnck_error_trap_push();
+	devilspie2_error_trap_push();
 	property = NULL;
 	result = XGetWindowProperty (gdk_x11_get_default_xdisplay (),
-	                             xwindow, atom,
-	                             0, G_MAXLONG,
-	                             False, AnyPropertyType, &type, &format, &nitems,
-	                             &bytes_after, &property);
+											xwindow, atom,
+											0, G_MAXLONG,
+											False, AnyPropertyType, &type,
+											&format, &nitems,
+											&bytes_after, &property);
 
-	err = my_wnck_error_trap_pop ();
+	err = devilspie2_error_trap_pop ();
 	if (err != Success || result != Success)
 		return NULL;
 
@@ -266,7 +268,8 @@ char* my_wnck_get_string_property_latin1(Window xwindow, Atom atom)
 			prop_names = g_new (char *, nitems + 1);
 			prop_names[nitems] = NULL;
 			for (i=0; i < nitems; i++) {
-				prop_names[i] = XGetAtomName (gdk_x11_get_default_xdisplay (), *pp++);
+				prop_names[i] = XGetAtomName (gdk_x11_get_default_xdisplay (),
+					*pp++);
 			}
 			retval = g_strjoinv (", ", prop_names);
 			for (i=0; i < nitems; i++) {
@@ -293,7 +296,9 @@ char* my_wnck_get_string_property_latin1(Window xwindow, Atom atom)
 }
 
 
-
+/**
+ *
+ */
 gboolean
 my_wnck_get_cardinal_list (Window xwindow, Atom atom,
                           gulong **cardinals, int *len)
@@ -305,19 +310,19 @@ my_wnck_get_cardinal_list (Window xwindow, Atom atom,
 	gulong *nums;
 	int err, result;
 
-	*cardinals=NULL;
-	*len=0;
+	*cardinals = NULL;
+	*len = 0;
 
-	my_wnck_error_trap_push();
-	type=None;
-	result=XGetWindowProperty(gdk_x11_get_default_xdisplay (),
+	devilspie2_error_trap_push();
+	type = None;
+	result = XGetWindowProperty(gdk_x11_get_default_xdisplay (),
 	                             xwindow,
 	                             atom,
 	                             0, G_MAXLONG,
 	                             False, XA_CARDINAL, &type, &format, &nitems,
 	                             &bytes_after, (void*)&nums);
 
-	err=my_wnck_error_trap_pop();
+	err = devilspie2_error_trap_pop();
 
 	if ((err!=Success) || (result!=Success))
 		return FALSE;
@@ -327,9 +332,9 @@ my_wnck_get_cardinal_list (Window xwindow, Atom atom,
 		return FALSE;
 	}
 
-	*cardinals=g_new(gulong, nitems);
+	*cardinals = g_new(gulong, nitems);
 	memcpy(*cardinals, nums, sizeof (gulong) * nitems);
-	*len=nitems;
+	*len = nitems;
 
 	XFree(nums);
 
@@ -340,60 +345,101 @@ my_wnck_get_cardinal_list (Window xwindow, Atom atom,
 /**
  *
  */
-glong my_wnck_get_cardinal(Window xwindow, Atom atom)
+int devilspie2_get_viewport_start(Window xid)
 {
-	Atom type;
-	int format;
-	gulong nitems;
-	gulong bytes_after;
-	gulong *nums;
-	glong data;
-	int err,result;
+	gulong *list;
+	int len;
 
-	my_wnck_error_trap_push();
-	type=None;
-	result=XGetWindowProperty(gdk_x11_get_default_xdisplay(),
-	                          xwindow,
-	                          atom,
-	                          0, G_MAXLONG,
-	                          False, XA_CARDINAL, &type, &format, &nitems,
-	                          &bytes_after, (void*)&nums);
+	int result = -1;
 
-	if (result!=Success)
-		return -1;
+	my_wnck_get_cardinal_list(RootWindowOfScreen(
+										devilspie2_window_get_xscreen(xid)),
+										my_wnck_atom_get("_NET_DESKTOP_VIEWPORT"),
+										&list, &len);
 
-	err=my_wnck_error_trap_pop();
-	if (err!=Success)
-		return -1;
+	if (len>0) result = list[0];
 
-	if (type!=XA_CARDINAL) {
-		XFree(nums);
-		return -1;
-	}
+	g_free(list);
 
-	data=nums[0];
-	XFree(nums);
-
-	return data;
+	return result;
 }
 
 
 /**
  *
  */
-int my_wnck_get_viewport_start(WnckWindow *win)
+void my_window_set_window_type(Window xid, gchar *window_type)
 {
-	gulong *list;
-	int len;
+	Display *display = gdk_x11_get_default_xdisplay();
 
-	int result=-1;
+	Atom atoms[10];
 
-	my_wnck_get_cardinal_list(RootWindowOfScreen(my_wnck_window_get_xscreen(win)),
-	                          my_wnck_atom_get("_NET_DESKTOP_VIEWPORT"), &list, &len);
+	/*
+	_NET_WM_WINDOW_TYPE_DESKTOP, ATOM
+	_NET_WM_WINDOW_TYPE_DOCK, ATOM
+	_NET_WM_WINDOW_TYPE_TOOLBAR, ATOM
+	_NET_WM_WINDOW_TYPE_MENU, ATOM
+	_NET_WM_WINDOW_TYPE_UTILITY, ATOM
+	_NET_WM_WINDOW_TYPE_SPLASH, ATOM
+	_NET_WM_WINDOW_TYPE_DIALOG, ATOM
+	_NET_WM_WINDOW_TYPE_NORMAL, ATOM
+	*/
 
-	if (len>0) result=list[0];
+	gchar *type = NULL;
 
-	g_free(list);
+	//	Make it a recognized _NET_WM_TYPE
+
+	if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_DESKTOP")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_DESKTOP");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_DOCK")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_DOCK");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_TOOLBAR")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_TOOLBAR");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_MENU")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_MENU");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_UTILITY")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_UTILITY");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_SPLASH")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_SPLASH");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_DIALOG")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_DIALOG");
+
+	} else if (g_ascii_strcasecmp(window_type, "WINDOW_TYPE_NORMAL")==0) {
+		type = g_strdup("_NET_WM_WINDOW_TYPE_NORMAL");
+
+	} else {
+		type = g_strdup(window_type);
+	}
+
+	atoms[0] = XInternAtom(display, type, False);
+
+	XChangeProperty(gdk_x11_get_default_xdisplay(), xid,
+						XInternAtom(display, "_NET_WM_WINDOW_TYPE", False), XA_ATOM, 32,
+						PropModeReplace, (unsigned char *) &atoms, 1);
+
+	if (type) g_free(type);
+}
+
+
+/**
+ *
+ */
+void my_window_set_opacity(Window xid, double value)
+{
+	Display *display = gdk_x11_get_default_xdisplay();
+
+	unsigned int opacity = (uint)(0xffffffff * value);
+	Atom atom_net_wm_opacity = XInternAtom(display, "_NET_WM_WINDOW_OPACITY", False);
+
+
+	XChangeProperty(gdk_x11_get_default_xdisplay(), xid,
+						atom_net_wm_opacity, XA_CARDINAL, 32,
+						PropModeReplace, (unsigned char *) &opacity, 1L);
 
-	return result;
 }
